@@ -2,8 +2,6 @@
  * viking -- GPS Data and Topo Analyzer, Explorer, and Manager
  *
  * Copyright (C) 2003-2005, Evan Battaglia <gtoevan@gmx.net>
- *
- * print.c
  * Copyright (C) 2007, Quy Tonthat <qtonthat@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -98,7 +96,7 @@ void a_print(VikWindow *vw, VikViewport *vvp)
 
   data.xres = data.yres = 1; // This forces it to default to a 100% page size
 
-  if (print_settings != NULL) 
+  if (print_settings != NULL)
     gtk_print_operation_set_print_settings (print_oper, print_settings);
 
   g_signal_connect (print_oper, "begin_print", G_CALLBACK (begin_print), &data);
@@ -107,17 +105,25 @@ void a_print(VikWindow *vw, VikViewport *vvp)
   g_signal_connect (print_oper, "create-custom-widget", G_CALLBACK (create_custom_widget_cb), &data);
 
   gtk_print_operation_set_custom_tab_label (print_oper, _("Image Settings"));
+  gtk_print_operation_set_embed_page_setup (print_oper, TRUE);
+  gtk_print_operation_set_show_progress (print_oper, TRUE);
 
+  GError *error = NULL;
   res = gtk_print_operation_run (print_oper,
                                  GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
-                                 GTK_WINDOW (vw), NULL);
+                                 GTK_WINDOW (vw),
+                                 &error);
 
   if (res == GTK_PRINT_OPERATION_RESULT_APPLY) {
     if (print_settings != NULL)
       g_object_unref (print_settings);
     print_settings = g_object_ref (gtk_print_operation_get_print_settings (print_oper));
   }
-
+  else if (res == GTK_PRINT_OPERATION_RESULT_ERROR) {
+    a_dialog_error_msg ( GTK_WINDOW(vw),
+                         error->message );
+    g_error_free ( error );
+  }
   g_object_unref (print_oper);
 }
 
@@ -125,18 +131,18 @@ static void begin_print(GtkPrintOperation *operation,
                         GtkPrintContext   *context,
                         PrintData         *data)
 {
-  // fputs("DEBUG: begin_print() called\n", stderr);
+  if ( vik_verbose )
+    g_debug ( "%s:", __FUNCTION__ );
   gtk_print_operation_set_n_pages (operation, data->num_pages);
   gtk_print_operation_set_use_full_page (operation, data->use_full_page);
-
 }
 
 static void end_print(GtkPrintOperation *operation,
                       GtkPrintContext   *context,
                       PrintData *data)
 {
-  // fputs("DEBUG: end_print() called\n", stderr);
-
+  if ( vik_verbose )
+    g_debug ( "%s:", __FUNCTION__ );
 }
 
 static void copy_row_from_rgb(guchar *surface_pixels, guchar *pixbuf_pixels, gint width)
@@ -181,7 +187,7 @@ static void copy_row_from_rgba(guchar *surface_pixels, guchar *pixbuf_pixels, gi
 static void draw_page_cairo(GtkPrintContext *context, PrintData *data)
 {
   cairo_t         *cr;
-  GdkPixbuf       *pixbuf_to_draw; 
+  GdkPixbuf       *pixbuf_to_draw;
   cairo_surface_t *surface;
   guchar          *surface_pixels;
   guchar          *pixbuf_pixels;
@@ -196,16 +202,24 @@ static void draw_page_cairo(GtkPrintContext *context, PrintData *data)
 
   cr = gtk_print_context_get_cairo_context(context);
 #if GTK_CHECK_VERSION (3,0,0)
-  pixbuf_to_draw = gdk_pixbuf_get_from_window ( gtk_widget_get_window (GTK_WIDGET(data->vvp)),
-                                                0, 0, data->width, data->height );
+  // On Wayland gdk_pixbuf_get_from_window() may be disallowed/return NULL.
+  // Use the viewport's offscreen pixbuf which works on both X11 and Wayland.
+  pixbuf_to_draw = vik_viewport_get_pixbuf (data->vvp, data->width, data->height);
 #else
   pixbuf_to_draw = gdk_pixbuf_get_from_drawable(NULL,
                                GDK_DRAWABLE(vik_viewport_get_pixmap(data->vvp)),
                                NULL, 0, 0, 0, 0, data->width, data->height);
 #endif
+
+  if (!pixbuf_to_draw) {
+    g_warning("%s: failed to obtain viewport pixbuf for printing", __FUNCTION__);
+    // Nothing sensible to draw — return early.
+    return;
+  }
+
   surface = cairo_image_surface_create(CAIRO_FORMAT_RGB24,
                                        data->width, data->height);
-  
+
   cr_dpi_x  = gtk_print_context_get_dpi_x  (context);
   cr_dpi_y  = gtk_print_context_get_dpi_y  (context);
 
@@ -223,7 +237,9 @@ static void draw_page_cairo(GtkPrintContext *context, PrintData *data)
   pixbuf_stride = gdk_pixbuf_get_rowstride(pixbuf_to_draw);
   pixbuf_n_channels = gdk_pixbuf_get_n_channels(pixbuf_to_draw);
 
-  // fprintf(stderr, "DEBUG: %s() surface_pixels=%p pixbuf_pixels=%p size=%d surface_width=%d surface_height=%d stride=%d data_height=%d pixmap_stride=%d pixmap_nchannels=%d pixmap_bit_per_Sample=%d\n", __PRETTY_FUNCTION__, surface_pixels, pixbuf_pixels, stride * data->height, cairo_image_surface_get_width(surface), cairo_image_surface_get_height(surface), stride, data->height, gdk_pixbuf_get_rowstride(pixbuf_to_draw), gdk_pixbuf_get_n_channels(pixbuf_to_draw), gdk_pixbuf_get_bits_per_sample(pixbuf_to_draw));
+  if ( vik_verbose )
+    g_debug ( "%s: size=%d surface_width=%d surface_height=%d stride=%d data_height=%d pixbuf_stride=%d pixbuf_nchannels=%d pixbuf_bits_per_sample=%d",
+              __FUNCTION__, stride * data->height, cairo_image_surface_get_width(surface), cairo_image_surface_get_height(surface), stride, data->height, gdk_pixbuf_get_rowstride(pixbuf_to_draw), gdk_pixbuf_get_n_channels(pixbuf_to_draw), gdk_pixbuf_get_bits_per_sample(pixbuf_to_draw) );
 
   /* Assume the pixbuf has 8 bits per channel */
   for (y = 0; y < data->height; y++, surface_pixels += stride, pixbuf_pixels += pixbuf_stride) {
@@ -251,9 +267,9 @@ static void draw_page(GtkPrintOperation *print,
                       gint               page_nr,
                       PrintData         *data)
 {
-  // fprintf(stderr, "DEBUG: draw_page() page_nr=%d\n", page_nr);
+  if ( vik_verbose )
+    g_debug ( "%s: page_nr=%d", __FUNCTION__, page_nr );
   draw_page_cairo(context, data);
-
 }
 
 /*********************** page layout gui *********************/
@@ -266,6 +282,12 @@ typedef struct
   GtkWidget       *preview;
 } CustomWidgetInfo;
 
+static void update_custom_widget_cb(GtkPrintOperation *operation,
+                                    GtkWidget         *widget,
+                                    GtkPageSetup      *setup,
+                                    GtkPrintSettings  *settings,
+                                    CustomWidgetInfo  *info);
+
 enum
 {
   BOTTOM,
@@ -300,7 +322,6 @@ static void set_scale_value(CustomWidgetInfo *pinfo)
   gdouble height;
   gdouble ratio, ratio_w, ratio_h;
 
-
   get_page_dimensions (pinfo, &width, &height, GTK_UNIT_INCH);
   ratio_w = 100 * pinfo->data->width / pinfo->data->xres / width;
   ratio_h = 100 * pinfo->data->height / pinfo->data->yres / height;
@@ -337,7 +358,6 @@ static void update_page_setup (CustomWidgetInfo *pinfo)
   if (pinfo->preview)
     vik_print_preview_set_image_offsets (VIK_PRINT_PREVIEW (pinfo->preview),
                                      pinfo->data->offset_x, pinfo->data->offset_y);
-
 }
 
 static void page_setup_cb (GtkWidget *widget, CustomWidgetInfo *info)
@@ -348,6 +368,9 @@ static void page_setup_cb (GtkWidget *widget, CustomWidgetInfo *info)
   GtkPageSetup      *page_setup;
   GtkWidget         *toplevel;
 
+  if ( vik_verbose )
+    g_debug ( "%s:", __FUNCTION__ );
+
   toplevel = gtk_widget_get_toplevel (widget);
   if (! gtk_widget_is_toplevel (toplevel))
     toplevel = NULL;
@@ -367,7 +390,6 @@ static void page_setup_cb (GtkWidget *widget, CustomWidgetInfo *info)
                                      page_setup);
 
   update_page_setup (info);
-
 }
 
 static void full_page_toggled_cb (GtkWidget *widget, CustomWidgetInfo *pinfo)
@@ -405,7 +427,6 @@ static void preview_offsets_changed_cb (GtkWidget *widget,
   info->data->offset_y = offset_y;
 
   update_offsets (info);
-
 }
 
 static void get_page_dimensions (CustomWidgetInfo *info,
@@ -429,7 +450,6 @@ static void get_page_dimensions (CustomWidgetInfo *info,
     *page_width -= left_margin + right_margin;
     *page_height -= top_margin + bottom_margin;
   }
-
 }
 
 static void get_max_offsets (CustomWidgetInfo *info,
@@ -539,9 +559,13 @@ static GtkWidget *create_custom_widget_cb(GtkPrintOperation *operation, PrintDat
   GtkWidget    *label;
   GtkPageSetup *setup;
 
-  CustomWidgetInfo  *info = g_malloc0(sizeof(CustomWidgetInfo));
-  g_signal_connect_swapped (data->operation, _("done"), G_CALLBACK (custom_widgets_cleanup), info);
+  if ( vik_verbose )
+    g_debug ( "%s:", __FUNCTION__ );
 
+  CustomWidgetInfo  *info = g_malloc0(sizeof(CustomWidgetInfo));
+  g_signal_connect_swapped (data->operation, "done", G_CALLBACK (custom_widgets_cleanup), info);
+
+  g_signal_connect (data->operation, "update-custom-widget", G_CALLBACK (update_custom_widget_cb), info);
 
   info->data = data;
 
@@ -648,6 +672,34 @@ static GtkWidget *create_custom_widget_cb(GtkPrintOperation *operation, PrintDat
                                             offset_x_max, offset_y_max);
 
   set_scale_value(info);
-  
+
   return layout;
 }
+
+// This handles changes from the GTK inbuilt 'Page Setup' tab,
+//  so that we can apply them to our custom Image Settings tab
+// NB ATM haven't found a way to update the 'Page Setup' widgets in reverse.
+//  i.e. for changes made in the custom widget - see page_setup_cb()
+//  although they are applied to the print 'operation' values,
+//  it doesn't result to any update of 'Page Setup' widgets themselves :(
+//  with no obvious way to force an update
+//  (no apparent specific API function, nor access to the widgets signal an update).
+// However, its not that critical as once a user is on the 'Image Settings' tab
+//  with a working 'preview' all the controls are there so get immediate feedback of changes;
+//  and thus unlikely to switch back (and forth) with the Page Setup tab.
+static void update_custom_widget_cb(GtkPrintOperation *operation,
+                                    GtkWidget         *widget,
+                                    GtkPageSetup      *setup,
+                                    GtkPrintSettings  *settings,
+                                    CustomWidgetInfo  *info)
+{
+  if ( vik_verbose )
+    g_debug ( "%s:", __FUNCTION__ );
+
+  // Ensure new values are applied!
+  gtk_print_operation_set_default_page_setup (operation, setup);
+
+  vik_print_preview_set_page_setup (VIK_PRINT_PREVIEW (info->preview), setup);
+
+  update_page_setup (info);
+}
