@@ -41,7 +41,7 @@ static VikLayerParamData geoclue_layer_get_param ( VikGeoclueLayer *vgl, guint16
 static const gchar* geoclue_layer_tooltip ( VikGeoclueLayer *vgl );
 
 static void geoclue_layer_change_coord_mode ( VikGeoclueLayer *vgl, VikCoordMode mode );
-static void geoclue_layer_add_menu_items ( VikGeoclueLayer *vtl, GtkMenu *menu, gpointer vlp );
+static void geoclue_layer_add_menu_items ( VikGeoclueLayer *vtl, GtkMenu *menu, gpointer vlp, VikStdLayerMenuItem selection, GtkTreeIter *iter );
 
 typedef gpointer menu_array_layer[2];
 static void geoclue_empty_cb ( menu_array_layer values );
@@ -106,12 +106,14 @@ VikLayerInterface vik_geoclue_layer_interface = {
 
   geoclue_layer_params,
   NUM_PARAMS,
+  0, // Number of VIK_LAYER_NOT_IN_PROPERTIES
   NULL,
   0,
 
   VIK_MENU_ITEM_ALL,
 
   (VikLayerFuncCreate)                  geoclue_layer_create,
+  (VikLayerFuncGetNewName)              NULL,
   (VikLayerFuncRealize)                 geoclue_layer_realize,
   (VikLayerFuncPostRead)                geoclue_layer_post_read,
   (VikLayerFuncFree)                    geoclue_layer_free,
@@ -139,6 +141,7 @@ VikLayerInterface vik_geoclue_layer_interface = {
   (VikLayerFuncMarshall)                geoclue_layer_marshall,
   (VikLayerFuncUnmarshall)              geoclue_layer_unmarshall,
 
+  (VikLayerFuncParamsDefaultGroup)      NULL,
   (VikLayerFuncSetParam)                geoclue_layer_set_param,
   (VikLayerFuncGetParam)                geoclue_layer_get_param,
   (VikLayerFuncChangeParam)             NULL,
@@ -231,7 +234,7 @@ static const gchar* geoclue_layer_tooltip ( VikGeoclueLayer *vgl )
 static void geoclue_layer_marshall( VikGeoclueLayer *vgl, guint8 **data, guint *datalen )
 {
 	VikLayer *layer;
-	guint8 *ld; 
+	guint8 *ld;
 	guint ll;
 	GByteArray* b = g_byte_array_new ();
 	guint len;
@@ -264,7 +267,7 @@ static VikGeoclueLayer *geoclue_layer_unmarshall ( guint8 *data, guint len, VikV
 #define alm_next \
 	len -= sizeof(guint) + alm_size; \
 	data += sizeof(guint) + alm_size;
-  
+
 	VikGeoclueLayer *rv = geoclue_layer_new(vvp);
 
 	vik_layer_unmarshall_params ( VIK_LAYER(rv), data+sizeof(guint), alm_size, vvp );
@@ -362,35 +365,9 @@ static void tracking_draw ( VikGeoclueLayer *vgl, VikViewport *vp )
 
 static void geoclue_layer_draw ( VikGeoclueLayer *vgl, VikViewport *vp )
 {
-	// NB I don't understand this half drawn business
-	// This is just copied from vikgpslayer.c
-	VikLayer *vl;
-	VikLayer *trigger = VIK_LAYER(vik_viewport_get_trigger( vp ));
-
-	vl = VIK_LAYER(vgl->trw);
-	if (vl == trigger) {
-		if ( vik_viewport_get_half_drawn ( vp ) ) {
-			vik_viewport_set_half_drawn ( vp, FALSE );
-			vik_viewport_snapshot_load( vp );
-		} else {
-			vik_viewport_snapshot_save( vp );
-		}
-	}
-	if ( !vik_viewport_get_half_drawn(vp) )
-		vik_layer_draw ( vl, vp );
-
-	if ( vgl->tracking ) {
-		if ( VIK_LAYER(vgl) == trigger ) {
-			if ( vik_viewport_get_half_drawn ( vp ) ) {
-				vik_viewport_set_half_drawn ( vp, FALSE );
-				vik_viewport_snapshot_load ( vp );
-			} else {
-				vik_viewport_snapshot_save ( vp );
-			}
-		}
-		if ( !vik_viewport_get_half_drawn(vp) )
-			tracking_draw ( vgl, vp );
-  }
+	VikLayer *vl = VIK_LAYER(vgl->trw);
+	vik_layer_draw ( vl, vp );
+	tracking_draw ( vgl, vp );
 }
 
 static void gcs_free ( VikGeoclueLayer *vgl )
@@ -410,7 +387,7 @@ static void geoclue_layer_change_coord_mode ( VikGeoclueLayer *vgl, VikCoordMode
 	vik_layer_change_coord_mode ( VIK_LAYER(vgl->trw), mode );
 }
 
-static void geoclue_layer_add_menu_items ( VikGeoclueLayer *vgl, GtkMenu *menu, gpointer vlp )
+static void geoclue_layer_add_menu_items ( VikGeoclueLayer *vgl, GtkMenu *menu, gpointer vlp, VikStdLayerMenuItem selection, GtkTreeIter *iter )
 {
 	static gpointer values[2];
 	GtkWidget *item;
@@ -482,6 +459,7 @@ static void geoclue_empty_cb ( menu_array_layer values )
 								_("Are you sure you want to delete geoclue data?"),
 								NULL ) )
 		return;
+	vgl->track = NULL;
 	vik_trw_layer_delete_all_waypoints ( vgl->trw );
 	vik_trw_layer_delete_all_tracks ( vgl->trw );
 }
@@ -618,9 +596,19 @@ notify_location ( GClueSimple *simple,
 		  update_all = FALSE;
 	}
 
+	// Create track if necessary
+	if ( vgl->record && vgl->track == NULL ) {
+		VikTrwLayer *vtl = vgl->trw;
+		vgl->track = vik_track_new();
+		vgl->track->has_color = TRUE;
+		gchar *name = make_track_name ( vtl );
+		vik_trw_layer_add_track ( vtl, name, vgl->track );
+		g_free ( name );
+	}
+
 	vgl->trkpt = vgl->record ? create_trackpoint ( vgl, vgl->coord, location ) : NULL;
 	vgl->first_trackpoint = FALSE;
-	
+
 	if ( vgl->trkpt ) {
 		if ( vgl->update_statusbar )
 			update_statusbar ( vgl, vw );
@@ -682,14 +670,6 @@ on_simple_ready ( GObject      *source_object,
 		libgeoclue_print_location ( location );
 	}
 
-	if ( vgl->record ) {
-		VikTrwLayer *vtl = vgl->trw;
-		vgl->track = vik_track_new();
-		gchar *name = make_track_name ( vtl );
-		vik_trw_layer_add_track ( vtl, name, vgl->track );
-		g_free ( name );
-	}
-
 	vgl->first_trackpoint = TRUE;
 
 	notify_location ( vgl->simple, NULL, vgl );
