@@ -42,6 +42,7 @@ VikLayerParam georef_layer_params[] = {
   { VIK_LAYER_GEOREF, "mpp_northing", VIK_LAYER_PARAM_DOUBLE, VIK_LAYER_NOT_IN_PROPERTIES, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL },
   { VIK_LAYER_GEOREF, "corner_zone", VIK_LAYER_PARAM_UINT, VIK_LAYER_NOT_IN_PROPERTIES, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL },
   { VIK_LAYER_GEOREF, "corner_letter_as_int", VIK_LAYER_PARAM_UINT, VIK_LAYER_NOT_IN_PROPERTIES, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL },
+  { VIK_LAYER_GEOREF, "rotation", VIK_LAYER_PARAM_DOUBLE, VIK_LAYER_NOT_IN_PROPERTIES, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL },
   { VIK_LAYER_GEOREF, "alpha", VIK_LAYER_PARAM_UINT, VIK_LAYER_NOT_IN_PROPERTIES, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL },
 };
 
@@ -53,6 +54,7 @@ enum {
   PARAM_MN,
   PARAM_CZ,
   PARAM_CL,
+  PARAM_RT,
   PARAM_AA,
   NUM_PARAMS };
 
@@ -66,7 +68,7 @@ static VikGeorefLayer *georef_layer_create ( VikViewport *vp );
 static void georef_layer_free ( VikGeorefLayer *vgl );
 static gboolean georef_layer_properties ( VikGeorefLayer *vgl, gpointer vp, gboolean have_apply );
 static void georef_layer_draw ( VikGeorefLayer *vgl, VikViewport *vp );
-static void georef_layer_add_menu_items ( VikGeorefLayer *vgl, GtkMenu *menu, gpointer vlp );
+static void georef_layer_add_menu_items ( VikGeorefLayer *vgl, GtkMenu *menu, gpointer vlp, VikStdLayerMenuItem selection, GtkTreeIter *iter );
 static void georef_layer_set_image ( VikGeorefLayer *vgl, const gchar *image );
 static gboolean georef_layer_dialog ( VikGeorefLayer *vgl, gpointer vp, GtkWindow *w, gboolean have_apply_button );
 static void georef_layer_load_image ( VikGeorefLayer *vgl, VikViewport *vp, gboolean from_file );
@@ -110,12 +112,14 @@ VikLayerInterface vik_georef_layer_interface = {
 
   georef_layer_params,
   NUM_PARAMS,
+  0, // Number of VIK_LAYER_NOT_IN_PROPERTIES
   NULL,
   0,
 
   VIK_MENU_ITEM_ALL,
 
   (VikLayerFuncCreate)                  georef_layer_create,
+  (VikLayerFuncGetNewName)              NULL,
   (VikLayerFuncRealize)                 NULL,
   (VikLayerFuncPostRead)                georef_layer_load_image,
   (VikLayerFuncFree)                    georef_layer_free,
@@ -143,6 +147,7 @@ VikLayerInterface vik_georef_layer_interface = {
   (VikLayerFuncMarshall)		georef_layer_marshall,
   (VikLayerFuncUnmarshall)		georef_layer_unmarshall,
 
+  (VikLayerFuncParamsDefaultGroup)      NULL,
   (VikLayerFuncSetParam)                georef_layer_set_param,
   (VikLayerFuncGetParam)                georef_layer_get_param,
   (VikLayerFuncChangeParam)             NULL,
@@ -168,6 +173,8 @@ VikLayerInterface vik_georef_layer_interface = {
 typedef struct {
   GtkWidget *x_spin;
   GtkWidget *y_spin;
+  GtkWidget *r_spin; // Rotation
+
   // UTM widgets
   GtkWidget *ce_spin; // top left
   GtkWidget *cn_spin; //    "
@@ -193,12 +200,27 @@ struct _VikGeorefLayer {
   gdouble mpp_easting, mpp_northing;
   struct LatLon ll_br; // Bottom Right
   guint width, height;
+  gdouble rotation; // Degrees
 
+  GdkPixbuf *rotated;
   GdkPixbuf *scaled;
-  guint32 scaled_width, scaled_height;
 
   gint click_x, click_y;
   changeable_widgets cw;
+
+  // (re)drawing values maintained between updates
+  //   to determine and use for current pixbuf being reused
+  gdouble old_rtn;
+  gint rtn_x_offset_full;
+  gint rtn_y_offset_full;
+  gint rtn_x_offset_scaled;
+  gint rtn_y_offset_scaled;
+  gboolean can_draw_reuse;
+  // Possibly don't need to monitor scales as lws,lhs should all change in unison
+  gdouble old_x_scale;
+  gdouble old_y_scale;
+  guint old_lws;
+  guint old_lhs;
 };
 
 static VikLayerParam io_prefs[] = {
@@ -267,6 +289,7 @@ static gboolean georef_layer_set_param ( VikGeorefLayer *vgl, VikLayerSetParam *
     case PARAM_ME: vgl->mpp_easting = vlsp->data.d; break;
     case PARAM_CZ: if ( vlsp->data.u <= 60 ) vgl->corner.zone = vlsp->data.u; break;
     case PARAM_CL: if ( vlsp->data.u >= 65 && vlsp->data.u <= 90 ) vgl->corner.letter = vlsp->data.u; break;
+    case PARAM_RT: vgl->rotation = vlsp->data.d; break;
     case PARAM_AA: if ( vlsp->data.u <= 255 ) vgl->alpha = vlsp->data.u; break;
 /*
   case PARAM_IMAGE: {
@@ -296,6 +319,9 @@ static gboolean georef_layer_set_param ( VikGeorefLayer *vgl, VikLayerSetParam *
       if ( vlsp->data.u >= 65 && vlsp->data.u <= 90 )
         changed = vik_layer_param_change_gchar ( vlsp->data, &vgl->corner.letter );
       break;
+    case PARAM_RT:
+      changed = vik_layer_param_change_double ( vlsp->data, &vgl->rotation );
+      break;
     case PARAM_AA:
       if ( vlsp->data.u <= 255 )
         changed = vik_layer_param_change_uint8 ( vlsp->data, &vgl->alpha );
@@ -354,6 +380,7 @@ static VikLayerParamData georef_layer_get_param ( VikGeorefLayer *vgl, guint16 i
     case PARAM_ME: rv.d = vgl->mpp_easting; break;
     case PARAM_CZ: rv.u = vgl->corner.zone; break;
     case PARAM_CL: rv.u = vgl->corner.letter; break;
+    case PARAM_RT: rv.d = vgl->rotation; break;
     case PARAM_AA: rv.u = vgl->alpha; break;
     default: break;
   }
@@ -373,17 +400,12 @@ static VikGeorefLayer *georef_layer_new ( VikViewport *vvp )
   vgl->mpp_northing = vik_viewport_get_ympp ( vvp );
   vgl->mpp_easting = vik_viewport_get_xmpp ( vvp );
   vik_coord_to_utm ( vik_viewport_get_center ( vvp ), &(vgl->corner) );
-
-  vgl->image = NULL;
-  vgl->pixbuf = NULL;
   vgl->click_x = -1;
   vgl->click_y = -1;
-  vgl->scaled = NULL;
-  vgl->scaled_width = 0;
-  vgl->scaled_height = 0;
   vgl->ll_br.lat = 0.0;
   vgl->ll_br.lon = 0.0;
   vgl->alpha = 255;
+  // Everything else is 0, FALSE or NULL
   return vgl;
 }
 
@@ -439,56 +461,159 @@ static void georef_layer_draw ( VikGeorefLayer *vgl, VikViewport *vp )
 {
   if ( vgl->pixbuf )
   {
-    gdouble xmpp = vik_viewport_get_xmpp(vp), ympp = vik_viewport_get_ympp(vp);
-    GdkPixbuf *pixbuf = vgl->pixbuf;
-    guint layer_width = vgl->width;
-    guint layer_height = vgl->height;
-
-    guint width = vik_viewport_get_width(vp), height = vik_viewport_get_height(vp);
-    gint32 x, y;
+    const gdouble xmpp = vik_viewport_get_xmpp(vp);
+    const gdouble ympp = vik_viewport_get_ympp(vp);
+    const guint vp_width = vik_viewport_get_width(vp);
+    const guint vp_height = vik_viewport_get_height(vp);
+    gint x, y;
     VikCoord corner_coord;
     vik_coord_load_from_utm ( &corner_coord, vik_viewport_get_coord_mode(vp), &(vgl->corner) );
     vik_viewport_coord_to_screen ( vp, &corner_coord, &x, &y );
 
-    /* mark to scale the pixbuf if it doesn't match our dimensions */
-    gboolean scale = FALSE;
-    if ( xmpp != vgl->mpp_easting || ympp != vgl->mpp_northing )
-    {
-      scale = TRUE;
-      layer_width = round(vgl->width * vgl->mpp_easting / xmpp);
-      layer_height = round(vgl->height * vgl->mpp_northing / ympp);
+    // The main point is to avoid the relatively compute expensive pixbuf operations: scaling and/or rotation
+    // otherwise for e.g. panning around or other redraw events we may be able use the existing pixbuf as is
+    //  but just with updated positioning within the viewport,
+    //  although this only really works when the entire image is in bounds.
 
-      // Has the scaling worked?
-      if ( layer_width == 0 || layer_height == 0 )
-	return;
-    }
+    const guint layer_width_scaled = round(gdk_pixbuf_get_width(vgl->pixbuf) * vgl->mpp_easting / xmpp);
+    const guint layer_height_scaled = round(gdk_pixbuf_get_height(vgl->pixbuf) * vgl->mpp_northing / ympp);
 
-    // If image not in viewport bounds - no need to draw it (or bother with any scaling)
-    if ( (x < 0 || x < width) && (y < 0 || y < height) && x+layer_width > 0 && y+layer_height > 0 ) {
+    // Has the scaling calculation worked?
+    // unclear if this can fail, but maintain defensive check
+    if ( layer_width_scaled == 0 || layer_height_scaled == 0 )
+      return;
 
-      if ( scale )
-      {
-        /* rescale if necessary */
-        if (layer_width == vgl->scaled_width && layer_height == vgl->scaled_height && vgl->scaled != NULL)
-          pixbuf = vgl->scaled;
-        else
-        {
-          pixbuf = gdk_pixbuf_scale_simple(
-            vgl->pixbuf,
-            layer_width,
-            layer_height,
-            GDK_INTERP_BILINEAR
-          );
+    const gdouble xscale = xmpp / vgl->mpp_easting; // source pixels per viewport pixel
+    const gdouble yscale = ympp / vgl->mpp_northing;
 
-          if (vgl->scaled != NULL)
-            g_object_unref(vgl->scaled);
+    const gboolean rotation_change = util_gdouble_different ( vgl->rotation, vgl->old_rtn );
+    vgl->old_rtn = vgl->rotation;
 
-          vgl->scaled = pixbuf;
-          vgl->scaled_width = layer_width;
-          vgl->scaled_height = layer_height;
+    // If there's rotation then the simple box [] needs expanding...
+
+    // NB rtn_x_offset and rtn_y_offset are in 'full' pixels
+    //  thus when applied to current viewport need to scaled too
+    if ( vgl->rotation != 0.0 ) {
+
+      if ( rotation_change ) {
+
+        if ( vgl->rotated )
+          g_object_unref ( vgl->rotated );
+        vgl->rotated = ui_pixbuf_rotate_full ( vgl->pixbuf, vgl->rotation );
+
+        if ( vgl->rotated ) {
+          if ( vgl->rotation < 0 ) {
+            vgl->rtn_y_offset_full = sin ( fabs(DEG2RAD(vgl->rotation)) ) * gdk_pixbuf_get_width(vgl->pixbuf);
+            vgl->rtn_x_offset_full = 0;
+          }
+          else {
+            vgl->rtn_x_offset_full = gdk_pixbuf_get_width(vgl->rotated) - (cos ( DEG2RAD(vgl->rotation) ) * gdk_pixbuf_get_width(vgl->pixbuf));
+            vgl->rtn_y_offset_full = 0;
+          }
+
+          vgl->width  = gdk_pixbuf_get_width ( vgl->rotated );
+          vgl->height = gdk_pixbuf_get_height ( vgl->rotated );
         }
       }
-      vik_viewport_draw_pixbuf ( vp, pixbuf, 0, 0, x, y, layer_width, layer_height ); /* todo: draw only what we need to. */
+    } else {
+      // No rotation, ensure clean-up and basic defaults
+      if ( vgl->rotated )
+        g_object_unref ( vgl->rotated );
+      vgl->rotated = NULL;
+
+      vgl->rtn_x_offset_full = 0;
+      vgl->rtn_y_offset_full = 0;
+
+      vgl->width = gdk_pixbuf_get_width(vgl->pixbuf);
+      vgl->height = gdk_pixbuf_get_height(vgl->pixbuf);
+    }
+
+    // Adjust to current vp pixel scale
+    vgl->rtn_x_offset_scaled = round((gdouble)vgl->rtn_x_offset_full / xscale);
+    vgl->rtn_y_offset_scaled = round((gdouble)vgl->rtn_y_offset_full / yscale);
+
+    // Realign x,y if image has been rotated
+    x -= vgl->rtn_x_offset_scaled;
+    y -= vgl->rtn_y_offset_scaled;
+
+    // Since x,y is altered, the following includes handling a rotated image
+
+    // Rectangle inside raw image to copy from
+    const gint src_xoffset = round(MIN(MAX(0, -x * xscale), vgl->width));
+    const gint src_yoffset = round(MIN(MAX(0, -y * yscale), vgl->height));
+    const gint src_width = round(MIN(vgl->width - src_xoffset, vp_width*xscale));
+    const gint src_height = round(MIN(vgl->height - src_yoffset, vp_height*yscale));
+
+    // Rectangle inside viewport to copy to
+    const gint vp_width_copy = round(MIN((vgl->width - src_xoffset)/xscale, vp_width));
+    const gint vp_height_copy = round(MIN((vgl->height - src_yoffset)/yscale, vp_height));
+    const gint vp_xoffset = MIN(MAX(0, x), vp_width);
+    const gint vp_yoffset = MIN(MAX(0, y), vp_height);
+
+    // Ensure fully expand to avoid implicit conversions and/or operator/expression ordering
+    const gboolean is_in_extended_viewport_bounds =
+      (x < ((gint)vp_width+vgl->rtn_x_offset_scaled)) && (y < ((gint)vp_height+vgl->rtn_y_offset_scaled)) &&
+      ((x+(gint)layer_width_scaled+vgl->rtn_x_offset_scaled) > 0) && ((y+(gint)layer_height_scaled+vgl->rtn_y_offset_scaled) > 0);
+
+    if ( is_in_extended_viewport_bounds ) {
+
+      // Only actually need to confirm that the top left is in viewport bounds;
+      //  since vik_viewport_draw_pixbuf() will handle out of bounds drawing
+      //  we can still use the existing image even if it overflows the width or height 'limits'
+      const gboolean is_top_left_in_viewport_bounds =
+        ( x >= 0 && x < (gint)vp_width && y >= 0 && y < (gint)vp_height );
+
+      const gboolean scale_pxbuf_change = ( util_gdouble_different(vgl->old_x_scale, xscale) ||
+                                            util_gdouble_different(vgl->old_y_scale, yscale) ||
+                                            (vgl->old_lws != layer_width_scaled) ||
+                                            (vgl->old_lhs != layer_height_scaled) );
+      vgl->old_lws = layer_width_scaled;
+      vgl->old_lhs = layer_height_scaled;
+      vgl->old_x_scale = xscale;
+      vgl->old_y_scale = yscale;
+
+      if ( is_top_left_in_viewport_bounds && !rotation_change && !scale_pxbuf_change ) {
+        // Check for transistions into the area
+        //  - can't reuse an old image as it may have parts missing as wasn't in previous viewport area
+        if ( vgl->can_draw_reuse ) {
+          vik_viewport_draw_pixbuf ( vp, vgl->scaled,
+                                     0, 0, vp_xoffset, vp_yoffset, vp_width_copy, vp_height_copy );
+          return;
+        }
+        // Could be alright next time!
+        vgl->can_draw_reuse = TRUE;
+      } else
+        vgl->can_draw_reuse = FALSE;
+
+      // Scaling to smaller than 2x2 seems to be broken
+      if ( vp_width_copy < 2 || vp_height_copy < 2 )
+        return;
+
+      // Otherwise create sub-region of source image to apply scaling to
+      GdkPixbuf *subpixbuf = gdk_pixbuf_new_subpixbuf ( vgl->rotated ? vgl->rotated : vgl->pixbuf,
+                                                        src_xoffset,
+                                                        src_yoffset,
+                                                        src_width,
+                                                        src_height );
+
+      GdkPixbuf *scld_pixbuf = ui_pixbuf_scale_simple_safe ( subpixbuf,
+                                                             vp_width_copy,
+                                                             vp_height_copy,
+                                                             50,
+                                                             GDK_INTERP_BILINEAR );
+      if ( subpixbuf != NULL )
+        g_object_unref ( subpixbuf );
+
+      if ( scld_pixbuf == NULL )
+        return;
+
+      // Save for reuse
+      if ( vgl->scaled )
+        g_object_unref ( vgl->scaled );
+      vgl->scaled = scld_pixbuf;
+
+      vik_viewport_draw_pixbuf ( vp, vgl->scaled,
+                                 0, 0, vp_xoffset, vp_yoffset, vp_width_copy, vp_height_copy );
     }
   }
 }
@@ -499,6 +624,8 @@ static void georef_layer_free ( VikGeorefLayer *vgl )
     g_free ( vgl->image );
   if ( vgl->scaled )
     g_object_unref ( vgl->scaled );
+  if ( vgl->rotated )
+    g_object_unref ( vgl->rotated );
   if ( vgl->pixbuf )
     g_object_unref ( vgl->pixbuf );
 }
@@ -511,7 +638,7 @@ static VikGeorefLayer *georef_layer_create ( VikViewport *vp )
 static gboolean georef_layer_properties ( VikGeorefLayer *vgl, gpointer vp, gboolean have_apply )
 {
   // TODO Determine if anything actually changed;
-  // For simplicity 
+  // For simplicity
   gboolean OKed = georef_layer_dialog ( vgl, vp, VIK_GTK_WINDOW_FROM_WIDGET(vp), have_apply );
   if ( OKed )
     vik_window_set_modified ( (VikWindow *)(VIK_GTK_WINDOW_FROM_WIDGET(vp)) );
@@ -531,6 +658,11 @@ static void georef_layer_load_image ( VikGeorefLayer *vgl, VikViewport *vp, gboo
     g_object_unref ( G_OBJECT(vgl->scaled) );
     vgl->scaled = NULL;
   }
+  if ( vgl->rotated )
+  {
+    g_object_unref ( G_OBJECT(vgl->rotated) );
+    vgl->rotated = NULL;
+  }
 
   vgl->pixbuf = gdk_pixbuf_new_from_file ( vgl->image, &gx );
 
@@ -560,6 +692,11 @@ static void georef_layer_set_image ( VikGeorefLayer *vgl, const gchar *image )
     g_object_unref ( vgl->scaled );
     vgl->scaled = NULL;
   }
+  if ( vgl->rotated )
+  {
+    g_object_unref ( vgl->rotated );
+    vgl->rotated = NULL;
+  }
   if ( image == NULL )
     vgl->image = NULL;
 
@@ -575,12 +712,17 @@ static void gdouble2spinwidget ( GtkWidget *widget, gdouble val )
   gtk_spin_button_set_value ( GTK_SPIN_BUTTON(widget), val > 0 ? val : -val );
 }
 
-static void set_widget_values ( changeable_widgets *cw, gdouble values[4] )
+// Supported World File Values
+//  only 5, since don't support both skew values
+#define SWFV 5
+
+static void set_widget_values ( changeable_widgets *cw, gdouble values[SWFV] )
 {
   gdouble2spinwidget ( cw->x_spin, values[0] );
-  gdouble2spinwidget ( cw->y_spin, values[1] );
-  gdouble2spinwidget ( cw->ce_spin, values[2] );
-  gdouble2spinwidget ( cw->cn_spin, values[3] );
+  gtk_spin_button_set_value ( GTK_SPIN_BUTTON(cw->r_spin), values[1] ); // -ve value allowed
+  gdouble2spinwidget ( cw->y_spin, values[2] );
+  gdouble2spinwidget ( cw->ce_spin, values[3] );
+  gdouble2spinwidget ( cw->cn_spin, values[4] );
 }
 
 static gboolean world_file_read_line ( FILE *ff, gdouble *value, gboolean use_value )
@@ -604,7 +746,7 @@ static gboolean world_file_read_line ( FILE *ff, gdouble *value, gboolean use_va
  *  x&y scale as meters per pixel
  *  x&y coords as UTM eastings and northings respectively
  */
-static gint world_file_read_file ( const gchar* filename, gdouble values[4] )
+static gint world_file_read_file ( const gchar* filename, gdouble values[SWFV] )
 {
   g_debug ("%s - trying world file %s", __FUNCTION__, filename);
 
@@ -613,18 +755,21 @@ static gint world_file_read_file ( const gchar* filename, gdouble values[4] )
     return 1;
   else {
     gint answer = 2; // Not enough info read yet
-    // **We do not handle 'skew' values ATM - normally they are a value of 0 anyway to align with the UTM grid
+    // **Only support one 'skew' value
     if ( world_file_read_line ( f, &values[0], TRUE ) // x scale
-      && world_file_read_line ( f, NULL, FALSE ) // Ignore value in y-skew line**
+      && world_file_read_line ( f, &values[1], TRUE ) // y-skew
       && world_file_read_line ( f, NULL, FALSE ) // Ignore value in x-skew line**
-      && world_file_read_line ( f, &values[1], TRUE ) // y scale
-      && world_file_read_line ( f, &values[2], TRUE ) // x-coordinate of the upper left pixel
-      && world_file_read_line ( f, &values[3], TRUE ) // y-coordinate of the upper left pixel
+      && world_file_read_line ( f, &values[2], TRUE ) // y scale
+      && world_file_read_line ( f, &values[3], TRUE ) // x-coordinate of the upper left pixel
+      && world_file_read_line ( f, &values[4], TRUE ) // y-coordinate of the upper left pixel
        )
     {
-       // Success
-       g_debug ("%s - %s - world file read success", __FUNCTION__, filename);
-       answer = 0;
+      if ( util_gdouble_different(values[1], 0.0) )
+        // Convert skew value (from pixels) into degrees
+        values[1] = RAD2DEG ( atan (values[1]/values[0]) );
+      // Success
+      g_debug ("%s - %s - world file read success", __FUNCTION__, filename);
+      answer = 0;
     }
     fclose ( f );
     return answer;
@@ -642,7 +787,7 @@ static void georef_layer_dialog_load ( changeable_widgets *cw )
 
   if ( gtk_dialog_run ( GTK_DIALOG ( file_selector ) ) == GTK_RESPONSE_ACCEPT )
   {
-     gdouble values[4];
+     gdouble values[SWFV];
      gchar *fn = gtk_file_chooser_get_filename ( GTK_FILE_CHOOSER(file_selector) );
      gint answer = world_file_read_file ( fn, values );
      if ( answer == 1 )
@@ -658,6 +803,7 @@ static void georef_layer_dialog_load ( changeable_widgets *cw )
   gtk_widget_destroy ( file_selector );
 }
 
+// Write World File
 static void georef_layer_export_params ( gpointer *pass_along[2] )
 {
   VikGeorefLayer *vgl = VIK_GEOREF_LAYER(pass_along[0]);
@@ -670,6 +816,12 @@ static void georef_layer_export_params ( gpointer *pass_along[2] )
   if ( gtk_dialog_run ( GTK_DIALOG ( file_selector ) ) == GTK_RESPONSE_ACCEPT )
   {
     gchar *fn = gtk_file_chooser_get_filename ( GTK_FILE_CHOOSER(file_selector) );
+    if ( g_file_test ( fn, G_FILE_TEST_EXISTS ) )
+      if ( ! a_dialog_yes_or_no ( GTK_WINDOW(file_selector), _("The file \"%s\" exists, do you wish to overwrite it?"), a_file_basename ( fn ) ) ) {
+        g_free ( fn );
+        gtk_widget_destroy ( file_selector );
+        return;
+      }
     FILE *f = g_fopen ( fn, "w" );
     g_free ( fn );
     gtk_widget_destroy ( file_selector );
@@ -680,7 +832,19 @@ static void georef_layer_export_params ( gpointer *pass_along[2] )
     }
     else
     {
-      fprintf ( f, "%f\n%f\n%f\n%f\n%f\n%f\n", vgl->mpp_easting, 0.0, 0.0, vgl->mpp_northing, vgl->corner.easting, vgl->corner.northing );
+      gdouble skewx = 0.0;
+      gdouble skewy = 0.0;
+      // Convert rotation to a skew value
+      // ATM we don't support separate x & y skew values.
+      if ( util_gdouble_different(vgl->rotation, 0.0) ) {
+        if ( util_gdouble_different(vgl->rotation, 90.0) ) {
+          skewx = -(vgl->mpp_easting)*tan(DEG2RAD(vgl->rotation));
+          skewy = -(vgl->mpp_northing)*tan(DEG2RAD(vgl->rotation));
+        }
+        else
+          g_message ( "%s: Warning can not export skews of 90 degrees rotation.", __FUNCTION__ );
+      }
+      fprintf ( f, "%f\n%f\n%f\n%f\n%f\n%f\n", vgl->mpp_easting, skewx, skewy, -vgl->mpp_northing, vgl->corner.easting, vgl->corner.northing );
       fclose ( f );
     }
   }
@@ -697,7 +861,7 @@ static gboolean maybe_read_world_file ( VikFileEntry *vfe, gpointer user_data )
 {
   if ( a_preferences_get (VIKING_PREFERENCES_IO_NAMESPACE "georef_auto_read_world_file")->b ) {
     const gchar* filename = vik_file_entry_get_filename(VIK_FILE_ENTRY(vfe));
-    gdouble values[4];
+    gdouble values[SWFV];
     if ( filename && user_data ) {
 
       changeable_widgets *cw = user_data;
@@ -869,7 +1033,8 @@ static gboolean georef_layer_dialog ( VikGeorefLayer *vgl, gpointer vp, GtkWindo
   /* Default to reject as user really needs to specify map file first */
   gtk_dialog_set_default_response ( GTK_DIALOG(dialog), GTK_RESPONSE_REJECT );
   GtkWidget *response_w = gtk_dialog_get_widget_for_response ( GTK_DIALOG(dialog), GTK_RESPONSE_REJECT );
-  GtkWidget *table, *wfp_hbox, *wfp_label, *wfp_button, *ce_label, *cn_label, *xlabel, *ylabel, *imagelabel;
+  GtkWidget *table, *wfp_hbox, *wfp_label, *wfp_button, *ce_label, *cn_label,
+    *xlabel, *ylabel, *rlabel, *imagelabel;
   changeable_widgets cw;
 
   GtkBox *dgbox = GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog)));
@@ -884,7 +1049,7 @@ static gboolean georef_layer_dialog ( VikGeorefLayer *vgl, gpointer vp, GtkWindo
   gtk_box_pack_start ( GTK_BOX(wfp_hbox), wfp_button, FALSE, FALSE, 3 );
 
   ce_label = gtk_label_new ( _("Corner pixel easting:") );
-  cw.ce_spin = gtk_spin_button_new ( (GtkAdjustment *) gtk_adjustment_new ( 4, -15000000.0, 1500000.0, 1, 5, 0 ), 1, 4 );
+  cw.ce_spin = gtk_spin_button_new ( (GtkAdjustment *) gtk_adjustment_new ( 4, -15000000.0, 15000000.0, 1, 5, 0 ), 1, 4 );
   gtk_widget_set_tooltip_text ( GTK_WIDGET(cw.ce_spin), _("the UTM \"easting\" value of the upper-left corner pixel of the map") );
 
   cn_label = gtk_label_new ( _("Corner pixel northing:") );
@@ -893,12 +1058,16 @@ static gboolean georef_layer_dialog ( VikGeorefLayer *vgl, gpointer vp, GtkWindo
 
   xlabel = gtk_label_new ( _("X (easting) scale (mpp): "));
   ylabel = gtk_label_new ( _("Y (northing) scale (mpp): "));
+  rlabel = gtk_label_new ( _("Rotation (degrees):"));
 
   cw.x_spin = gtk_spin_button_new ( (GtkAdjustment *) gtk_adjustment_new ( 4, VIK_VIEWPORT_MIN_ZOOM, VIK_VIEWPORT_MAX_ZOOM, 1, 5, 0 ), 1, 8 );
   gtk_widget_set_tooltip_text ( GTK_WIDGET(cw.x_spin), _("the scale of the map in the X direction (meters per pixel)") );
   cw.y_spin = gtk_spin_button_new ( (GtkAdjustment *) gtk_adjustment_new ( 4, VIK_VIEWPORT_MIN_ZOOM, VIK_VIEWPORT_MAX_ZOOM, 1, 5, 0 ), 1, 8 );
   gtk_widget_set_tooltip_text ( GTK_WIDGET(cw.y_spin), _("the scale of the map in the Y direction (meters per pixel)") );
 
+  cw.r_spin = gtk_spin_button_new ( (GtkAdjustment *) gtk_adjustment_new ( 0.0, -90.0, 90.0, 1, 5, 0 ), 1, 3 );
+  gtk_widget_set_tooltip_text ( GTK_WIDGET(cw.r_spin), _("Clockwise about the upper-left corner") );
+
   imagelabel = gtk_label_new ( _("Map Image:") );
   cw.imageentry = vik_file_entry_new (GTK_FILE_CHOOSER_ACTION_OPEN, VF_FILTER_IMAGE, maybe_read_world_file, &cw);
 
@@ -906,6 +1075,7 @@ static gboolean georef_layer_dialog ( VikGeorefLayer *vgl, gpointer vp, GtkWindo
   gtk_spin_button_set_value ( GTK_SPIN_BUTTON(cw.cn_spin), vgl->corner.northing );
   gtk_spin_button_set_value ( GTK_SPIN_BUTTON(cw.x_spin), vgl->mpp_easting );
   gtk_spin_button_set_value ( GTK_SPIN_BUTTON(cw.y_spin), vgl->mpp_northing );
+  gtk_spin_button_set_value ( GTK_SPIN_BUTTON(cw.r_spin), vgl->rotation );
   if ( vgl->image )
     vik_file_entry_set_filename ( VIK_FILE_ENTRY(cw.imageentry), vgl->image );
 
@@ -916,6 +1086,8 @@ static gboolean georef_layer_dialog ( VikGeorefLayer *vgl, gpointer vp, GtkWindo
   gtk_table_attach_defaults ( GTK_TABLE(table), cw.x_spin, 1, 2, 2, 3 );
   gtk_table_attach_defaults ( GTK_TABLE(table), ylabel, 0, 1, 3, 4 );
   gtk_table_attach_defaults ( GTK_TABLE(table), cw.y_spin, 1, 2, 3, 4 );
+  gtk_table_attach_defaults ( GTK_TABLE(table), rlabel, 0, 1, 4, 5 );
+  gtk_table_attach_defaults ( GTK_TABLE(table), cw.r_spin, 1, 2, 4, 5 );
 
   cw.tabs = gtk_notebook_new();
   GtkWidget *table_utm = gtk_table_new ( 3, 2, FALSE );
@@ -1020,14 +1192,13 @@ static gboolean georef_layer_dialog ( VikGeorefLayer *vgl, gpointer vp, GtkWindo
          vgl->corner.letter = toupper(*letter);
       vgl->mpp_easting = gtk_spin_button_get_value ( GTK_SPIN_BUTTON(cw.x_spin) );
       vgl->mpp_northing = gtk_spin_button_get_value ( GTK_SPIN_BUTTON(cw.y_spin) );
+      vgl->rotation = gtk_spin_button_get_value ( GTK_SPIN_BUTTON(cw.r_spin) );
       vgl->ll_br = get_ll_br (vgl);
       check_br_is_good_or_msg_user ( vgl );
       // TODO check if image has changed otherwise no need to regenerate pixbuf
-      if ( !vgl->pixbuf ) {
-        if ( g_strcmp0 (vgl->image, vik_file_entry_get_filename(VIK_FILE_ENTRY(cw.imageentry)) ) != 0 ) {
-          georef_layer_set_image ( vgl, vik_file_entry_get_filename(VIK_FILE_ENTRY(cw.imageentry)) );
-          georef_layer_load_image ( vgl, VIK_VIEWPORT(vp), FALSE );
-        }
+      if ( g_strcmp0 (vgl->image, vik_file_entry_get_filename(VIK_FILE_ENTRY(cw.imageentry)) ) != 0 ) {
+        georef_layer_set_image ( vgl, vik_file_entry_get_filename(VIK_FILE_ENTRY(cw.imageentry)) );
+        georef_layer_load_image ( vgl, VIK_VIEWPORT(vp), FALSE );
       }
 
       vgl->alpha = (guint8) gtk_range_get_value ( GTK_RANGE(alpha_scale) );
@@ -1035,6 +1206,8 @@ static gboolean georef_layer_dialog ( VikGeorefLayer *vgl, gpointer vp, GtkWindo
         vgl->pixbuf = ui_pixbuf_set_alpha ( vgl->pixbuf, vgl->alpha );
       if ( vgl->scaled && vgl->alpha <= 255 )
         vgl->scaled = ui_pixbuf_set_alpha ( vgl->scaled, vgl->alpha );
+      if ( vgl->rotated && vgl->alpha <= 255 )
+        vgl->rotated = ui_pixbuf_set_alpha ( vgl->rotated, vgl->alpha );
 
       a_settings_set_integer ( VIK_SETTINGS_GEOREF_TAB, gtk_notebook_get_current_page(GTK_NOTEBOOK(cw.tabs)) );
 
@@ -1075,7 +1248,7 @@ static void georef_layer_goto_center ( gpointer vgl_vlp[2] )
   vik_layers_panel_emit_update ( VIK_LAYERS_PANEL(vgl_vlp[1]), FALSE );
 }
 
-static void georef_layer_add_menu_items ( VikGeorefLayer *vgl, GtkMenu *menu, gpointer vlp )
+static void georef_layer_add_menu_items ( VikGeorefLayer *vgl, GtkMenu *menu, gpointer vlp, VikStdLayerMenuItem selection, GtkTreeIter *iter )
 {
   static gpointer pass_along[2];
   pass_along[0] = vgl;
