@@ -27,7 +27,7 @@
 #include "vikwebtool_datasource.h"
 #include <ctype.h>
 #include <string.h>
-
+#include <math.h>
 #include <glib.h>
 #include <glib/gi18n.h>
 
@@ -56,6 +56,7 @@ struct _VikWebtoolDatasourcePrivate
 	gchar *babel_filter_args;
     gchar *input_label;
 	gchar *user_string;
+	DownloadFileOptions options;
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (VikWebtoolDatasource, vik_webtool_datasource, VIK_WEBTOOL_TYPE)
@@ -68,7 +69,11 @@ enum
 	PROP_URL_FORMAT_CODE,
 	PROP_FILE_TYPE,
 	PROP_BABEL_FILTER_ARGS,
-    PROP_INPUT_LABEL
+	PROP_INPUT_LABEL,
+	PROP_REFERER,
+	PROP_USER_AGENT,
+	PROP_FOLLOW_LOCATION,
+	PROP_CUSTOM_HTTP_HEADERS,
 };
 
 static void webtool_datasource_set_property (GObject      *object,
@@ -79,7 +84,7 @@ static void webtool_datasource_set_property (GObject      *object,
 	VikWebtoolDatasourcePrivate *priv = WEBTOOL_DATASOURCE_GET_PRIVATE ( object );
 
 	switch ( property_id ) {
-	
+
     case PROP_URL:
 		g_free ( priv->url );
 		priv->url = g_value_dup_string ( value );
@@ -110,6 +115,32 @@ static void webtool_datasource_set_property (GObject      *object,
 		g_debug ( "VikWebtoolDatasource.input_label: %s", priv->input_label );
 		break;
 
+	case PROP_REFERER:
+		g_free (priv->options.referer);
+		priv->options.referer = g_value_dup_string (value);
+		break;
+
+	case PROP_USER_AGENT:
+		g_free (priv->options.user_agent);
+		priv->options.user_agent = g_value_dup_string (value);
+		break;
+
+    case PROP_FOLLOW_LOCATION:
+		priv->options.follow_location = g_value_get_long (value);
+		break;
+
+    case PROP_CUSTOM_HTTP_HEADERS:
+		{
+			g_free (priv->options.custom_http_headers);
+			const gchar *str = g_value_get_string ( value );
+			// Convert the literal characters '\'+'n' into actual newlines '\n'
+			if ( str )
+				priv->options.custom_http_headers = g_strcompress ( str );
+			else
+				priv->options.custom_http_headers = NULL;
+		}
+		break;
+
 	default:
 		/* We don't have any other property... */
 		G_OBJECT_WARN_INVALID_PROPERTY_ID ( object, property_id, pspec );
@@ -126,11 +157,15 @@ static void webtool_datasource_get_property (GObject    *object,
 
 	switch ( property_id ) {
 
-	case PROP_URL:               g_value_set_string ( value, priv->url ); break;
-	case PROP_URL_FORMAT_CODE:	 g_value_set_string ( value, priv->url_format_code ); break;
-	case PROP_FILE_TYPE:         g_value_set_string ( value, priv->url ); break;
-	case PROP_BABEL_FILTER_ARGS: g_value_set_string ( value, priv->babel_filter_args ); break;
-	case PROP_INPUT_LABEL:       g_value_set_string ( value, priv->input_label ); break;
+	case PROP_URL:                 g_value_set_string ( value, priv->url ); break;
+	case PROP_URL_FORMAT_CODE:     g_value_set_string ( value, priv->url_format_code ); break;
+	case PROP_FILE_TYPE:           g_value_set_string ( value, priv->url ); break;
+	case PROP_BABEL_FILTER_ARGS:   g_value_set_string ( value, priv->babel_filter_args ); break;
+	case PROP_INPUT_LABEL:         g_value_set_string ( value, priv->input_label ); break;
+	case PROP_REFERER:             g_value_set_string ( value, priv->options.referer ); break;
+	case PROP_USER_AGENT:          g_value_set_string ( value, priv->options.user_agent );break;
+	case PROP_FOLLOW_LOCATION:     g_value_set_long   ( value, priv->options.follow_location); break;
+	case PROP_CUSTOM_HTTP_HEADERS: g_value_set_string ( value, priv->options.custom_http_headers); break;
 
 	default:
 		/* We don't have any other property... */
@@ -149,10 +184,10 @@ typedef struct {
 
 static void ensure_last_user_strings_hash() {
     if ( last_user_strings == NULL ) {
-        last_user_strings = g_hash_table_new_full ( g_str_hash, 
+        last_user_strings = g_hash_table_new_full ( g_str_hash,
                                                     g_str_equal,
                                                     g_free,
-                                                    g_free ); 
+                                                    g_free );
     }
 }
 
@@ -168,8 +203,8 @@ static gchar *get_last_user_string ( const datasource_t *source ) {
 
 static void set_last_user_string ( const datasource_t *source, const gchar *s ) {
     ensure_last_user_strings_hash();
-    g_hash_table_insert ( last_user_strings, 
-                          vik_ext_tool_get_label ( source->self ), 
+    g_hash_table_insert ( last_user_strings,
+                          vik_ext_tool_get_label ( source->self ),
                           g_strdup ( s ) );
 }
 
@@ -246,8 +281,12 @@ static void datasource_get_process_options ( gpointer user_data, ProcessOptions
 		po->input_file_type = NULL;
 	g_strfreev ( parts );
 
-	options = NULL;
-	po->babel_filters = priv->babel_filter_args;
+	po->babel_filters = g_strdup ( priv->babel_filter_args );
+
+	options->referer             = priv->options.referer;
+	options->user_agent          = priv->options.user_agent;
+	options->follow_location     = priv->options.follow_location;
+	options->custom_http_headers = priv->options.custom_http_headers;
 }
 
 static void cleanup ( gpointer data )
@@ -335,7 +374,7 @@ static void vik_webtool_datasource_class_init ( VikWebtoolDatasourceClass *klass
 	pspec = g_param_spec_string ("babel_filter_args",
 	                             "The command line filter options to pass to gpsbabel",
 	                             "Set the command line filter options for gpsbabel",
-	                             NULL, // default value 
+	                             NULL, // default value
 	                             G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
 	g_object_class_install_property (gobject_class,
 	                                 PROP_BABEL_FILTER_ARGS,
@@ -350,6 +389,36 @@ static void vik_webtool_datasource_class_init ( VikWebtoolDatasourceClass *klass
 	                                 PROP_INPUT_LABEL,
 	                                 pspec);
 
+	pspec = g_param_spec_string ("referer",
+	                             "Referer",
+	                             "The REFERER string to use in HTTP request",
+	                             NULL, // default value
+	                             G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
+	g_object_class_install_property (gobject_class, PROP_REFERER, pspec);
+
+	pspec = g_param_spec_string ("user-agent",
+	                             "User Agent",
+	                             "The User Agent string to send in the request",
+	                             NULL, // default value
+	                             G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
+	g_object_class_install_property (gobject_class, PROP_USER_AGENT, pspec);
+
+	pspec = g_param_spec_long ("follow-location",
+	                           "Follow location",
+	                           "Specifies the number of retries to follow a redirect while downloading a URL",
+	                           -1, // minimum value (unlimited)
+	                           G_MAXLONG, // maximum value
+	                           -1, // default value - automatically follow location with unlimited redirects
+	                           G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
+	g_object_class_install_property (gobject_class, PROP_FOLLOW_LOCATION, pspec);
+
+	pspec = g_param_spec_string ("custom-http-headers",
+	                             "Custom HTTP Headers",
+	                             "Custom HTTP Headers, use '\n' to separate multiple headers",
+	                             NULL, // default value
+	                             G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
+	g_object_class_install_property (gobject_class, PROP_CUSTOM_HTTP_HEADERS, pspec);
+
 	parent_class = g_type_class_peek_parent (klass);
 
 	base_class = VIK_WEBTOOL_CLASS ( klass );
@@ -393,6 +462,14 @@ static void vik_webtool_datasource_init ( VikWebtoolDatasource *self )
 	priv->babel_filter_args = NULL;
     priv->input_label = NULL;
 	priv->user_string = NULL;
+	priv->options.referer = NULL;
+	priv->options.follow_location = -1;
+	priv->options.expiry_age = 0;
+	priv->options.check_file = NULL;
+	priv->options.check_file_server_time = FALSE;
+	priv->options.use_etag = FALSE;
+	priv->options.user_agent = NULL;
+	priv->options.custom_http_headers = NULL;
 }
 
 static void webtool_datasource_finalize ( GObject *gob )
@@ -404,6 +481,9 @@ static void webtool_datasource_finalize ( GObject *gob )
 	g_free ( priv->babel_filter_args ); priv->babel_filter_args = NULL;
 	g_free ( priv->input_label ); priv->input_label = NULL;
 	g_free ( priv->user_string); priv->user_string = NULL;
+	g_free ( priv->options.referer ); priv->options.referer = NULL;
+	g_free ( priv->options.user_agent ); priv->options.user_agent = NULL;
+	g_free ( priv->options.custom_http_headers ); priv->options.custom_http_headers = NULL;
 	G_OBJECT_CLASS(parent_class)->finalize(gob);
 }
 
@@ -420,28 +500,36 @@ static gchar *webtool_datasource_get_url ( VikWebtool *self, VikWindow *vw )
 
 	// Get top left and bottom right lat/lon pairs from the viewport
 	gdouble min_lat, max_lat, min_lon, max_lon;
-	gchar sminlon[G_ASCII_DTOSTR_BUF_SIZE];
-	gchar smaxlon[G_ASCII_DTOSTR_BUF_SIZE];
-	gchar sminlat[G_ASCII_DTOSTR_BUF_SIZE];
-	gchar smaxlat[G_ASCII_DTOSTR_BUF_SIZE];
+	gchar sminlon[COORDS_STR_BUFFER_SIZE];
+	gchar smaxlon[COORDS_STR_BUFFER_SIZE];
+	gchar sminlat[COORDS_STR_BUFFER_SIZE];
+	gchar smaxlat[COORDS_STR_BUFFER_SIZE];
 	vik_viewport_get_min_max_lat_lon ( viewport, &min_lat, &max_lat, &min_lon, &max_lon );
 
+	min_lon = ROUND_TO_DECIMAL_PLACES ( min_lon, 6 );
+	max_lon = ROUND_TO_DECIMAL_PLACES ( max_lon, 6 );
+	min_lat = ROUND_TO_DECIMAL_PLACES ( min_lat, 6 );
+	max_lat = ROUND_TO_DECIMAL_PLACES ( max_lat, 6 );
 	// Cannot simply use g_strdup_printf and gdouble due to locale.
 	// As we compute an URL, we have to think in C locale.
-	g_ascii_dtostr (sminlon, G_ASCII_DTOSTR_BUF_SIZE, min_lon);
-	g_ascii_dtostr (smaxlon, G_ASCII_DTOSTR_BUF_SIZE, max_lon);
-	g_ascii_dtostr (sminlat, G_ASCII_DTOSTR_BUF_SIZE, min_lat);
-	g_ascii_dtostr (smaxlat, G_ASCII_DTOSTR_BUF_SIZE, max_lat);
+	// Furthermore ensure decimal output (never scientific notation)
+	a_coords_dtostr_buffer ( min_lon, sminlon );
+	a_coords_dtostr_buffer ( max_lon, smaxlon );
+	a_coords_dtostr_buffer ( min_lat, sminlat );
+	a_coords_dtostr_buffer ( max_lat, smaxlat );
 
 	// Center values
 	const VikCoord *coord = vik_viewport_get_center ( viewport );
 	struct LatLon ll;
 	vik_coord_to_latlon ( coord, &ll );
 
-	gchar scenterlat[G_ASCII_DTOSTR_BUF_SIZE];
-	gchar scenterlon[G_ASCII_DTOSTR_BUF_SIZE];
-	g_ascii_dtostr (scenterlat, G_ASCII_DTOSTR_BUF_SIZE, ll.lat);
-	g_ascii_dtostr (scenterlon, G_ASCII_DTOSTR_BUF_SIZE, ll.lon);
+	ll.lat = ROUND_TO_DECIMAL_PLACES ( ll.lat, 6 );
+	ll.lon = ROUND_TO_DECIMAL_PLACES ( ll.lon, 6 );
+
+	gchar scenterlat[COORDS_STR_BUFFER_SIZE];
+	gchar scenterlon[COORDS_STR_BUFFER_SIZE];
+	a_coords_dtostr_buffer ( ll.lat, scenterlat );
+	a_coords_dtostr_buffer ( ll.lon, scenterlon );
 
 	guint8 zoom = 17; // A zoomed in default
 	// zoom - ideally x & y factors need to be the same otherwise use the default
@@ -482,7 +570,7 @@ static gchar *webtool_datasource_get_url ( VikWebtool *self, VikWindow *vw )
 	for ( i = 0; i < MAX_NUMBER_CODES; i++ ) {
 		g_free ( values[i] );
 	}
-	
+
 	return url;
 }
 
